@file:Suppress("unused")

package net.sergeych.synctools

import java.nio.channels.CompletionHandler
import kotlin.coroutines.Continuation
import kotlin.coroutines.resume
import kotlin.coroutines.resumeWithException

/**
 * Helper class to handle Java continuation with Kotlin coroutines.
 * Usage sample:
 * ```kotlin
 * val socket = withContext(Dispatchers.IO) {
 *     AsynchronousSocketChannel.open()
 * }
 * suspendCoroutine { cont ->
 *     socket.connect(address.socketAddress, cont, VoidCompletionHandler)
 * }
 * ```
 */
open class ContinuationHandler<T> : CompletionHandler<T, Continuation<T>> {
    override fun completed(result: T, attachment: Continuation<T>) {
        attachment.resume(result)
    }

    override fun failed(exc: Throwable, attachment: Continuation<T>) {
        attachment.resumeWithException(exc)
    }
}

object VoidCompletionHandler : ContinuationHandler<Void>()

object IntCompletionHandler : ContinuationHandler<Int>()